@@ -1,6 +1,7 @@
 from pathlib import Path
 from typing import Tuple, List, Optional
 from collections import OrderedDict
+from random import randrange
 
 from dovado_rtl.config import Configuration
 import dovado_rtl.vivado_interaction as vivado
@@ -21,8 +22,8 @@ from dovado_rtl.cli_utility import (
     validate_directives,
     validate_target_clock,
     validate_estimator,
-    validate_nearest_distance,
     validate_int_metrics,
+    validate_controller,
 )
 import typer
 
@@ -221,7 +222,13 @@ def points(
             out_file.open("a").writelines([output_line[:-1] + "\n"])
 
 
-@app.command("space")
+@app.command(
+    "space",
+    context_settings={
+        "allow_extra_args": True,
+        "ignore_unknown_options": True,
+    },
+)
 def space(
     ctx: typer.Context,
     param_ranges: List[int] = typer.Argument(
@@ -240,66 +247,72 @@ def space(
     optimization_runtime: Optional[str] = typer.Option(
         None, help="optimization timeout as hh:mm:ss"
     ),
-    approximate: bool = typer.Option(
+    approximate: Optional[bool] = typer.Option(
         False, help="enable approximations for fitness function"
     ),
-    #fitness_estimator: str = typer.Option(
-    #    "",
-    #    callback=validate_estimator,
-    #    help="estimator to use for fitness function approximation,"
-    #    + "'kernel_ridge' is default. 'stream_learning_isoup' selects a ISOUP Adaptive Tree Regressor trained incrementally "
-    #    + "while 'stream_learning_stacked' selects a Stacked tree regressor",
-    #),
-    #test_estimation: bool = typer.Option(
-    #    False,
-    #    help="record all estimated values along with the real "
-    #    + "computed values (slows down computation a lot)",
-    #),
-    #record_design_values: bool = typer.Option(
-    #    False, help="record all design values in a csv file"
-    #),
-    #read_design_values: bool = typer.Option(
-    #    False,
-    #    help="read design values from a csv file recorded with "
-    #    + "the --record-design-values flag",
-    #),
+    estimation_model: Optional[str] = typer.Option(
+        "HoeffdingAdaptiveTree",
+        callback=validate_estimator,
+        help="estimator to use for fitness function approximation (see movado docs for more information),"
+        + "'HoeffdingAdaptiveTree' is default. 'KernelRidge' selects a kernel regression",
+    ),
+    controller_model: Optional[str] = typer.Option(
+        "Distance",
+        callback=validate_controller,
+        help="controller to use for fitness function approximation (see movado docs for more information),"
+        + "'Distance' is default. 'Mab' selects a pure multi-armed bandit controller",
+    ),
+    record_design_values: bool = typer.Option(
+        False, help="record all design values in a csv file"
+    ),
+    read_design_values: bool = typer.Option(
+        False,
+        help="read design values from a csv file recorded with "
+        + "the --record-design-values flag",
+    ),
 ):
     """
     RTL design space exploration
     """
     it = iter(param_ranges)
     ranges_dict = OrderedDict(zip(ctx.obj["parameters"], zip(it, it)))
-    estimator = None
-    if approximate:
-        raise Exception("To be implemented")
-    else:
-        if param_initial_values:
-            design_value = ctx.obj["point_evaluator"].evaluate(
-                tuple(param_initial_values)
+    if param_initial_values:
+        design_value = ctx.obj["point_evaluator"].evaluate(
+            tuple(param_initial_values)
+        )
+        if not design_value:
+            raise Exception(
+                "Could not synthesize/implement " + str(param_initial_values)
             )
-            if not design_value:
-                raise Exception(
-                    "Could not synthesize/implement "
-                    + str(param_initial_values)
-                )
-        else:
-            raise Exception("To implement")
-            # first estimation is to retrieve available utilization metrics
+    else:
+        print(
+            "Trying random parameter values in the range given to get first synthesis/implementation"
+        )
+        design_value = ctx.obj["point_evaluator"].evaluate(
+            tuple([randrange(i, j) for i, j in ranges_dict.values()])
+        )
+        while not design_value:
+            print(
+                "Trying random parameter values in the range given to get first synthesis/implementation"
+            )
+            design_value = ctx.obj["point_evaluator"].evaluate(
+                tuple([randrange(i, j) for i, j in ranges_dict.values()])
+            )
+        print("First synthesis succeeded")
+
+    metrics = ctx.obj["point_evaluator"].get_metrics()
 
     fitness_evaluator = FitnessEvaluator(
         ctx.obj["point_evaluator"],
-        ctx.obj["config"],
+        approx=approximate,
+        control=controller_model,
+        estim=estimation_model,
+        **{
+            ctx.args[i].replace("--", ""): ctx.args[i + 1]
+            for i in range(0, len(ctx.args), 2)
+        }
     )
 
-    # TODO ask user for initial parameters so as to be sure to have a correct first synthesis/implementation
-    metrics = ctx.obj["point_evaluator"].get_metrics()
-
-    if not metrics:
-        raise Exception(
-            "INITIAL_SAMPLES must be at least one in order to analyze "
-            + "which utilization metrics are available for the given board"
-        )
-
     result = optimize(
         fitness_evaluator,
         ranges_dict,
